### 2.9
# Use the matrix inversion lemma (A.1) on page 241.
# Replace matrix B with column vector b, matrix D with row vector d^T, and
# matrix C with scalar 1. Inserting this, and noting that 
# (1 + d^t*A^{-1}*b) is a scalar, thus, the inverse is simply the reciprocal. 
# Then we end up with equation (A.2), the Sherman-Morrision formula.

### 2.11
# Prove (2.24). An exercise in algebra.
# See separate file .

### 2.12 
# The difference between the confidence interval of the value of the function 
# and the prediction interval, both relative to the next observation, is that
# the former yields an interval which says something about the location of the
# expected value of the function given the next observation, while the latter 
# gives a wider interval which also includes the noise uncertainty related to
# the new value.
# Informally speaking, the confidence interval says something about
# E[\hat{f}(X)|X = x_new], while the prediction interval says something about
# \hat{f}(x_new).

### 2.13
# The discrepancy between the curves of Figure 2.11 and 2.12, is that in the 
# former we only have a constant and linear term in the linear predictor eta.
# While in the latter figure, we have included a quadratic term. 
# A linear function is always monotone increasing or decreasing, while 
# a quadratic function is either convex or concave, hence, we get a 
# different behavior. play around with the numbers bellow.
a = 1
b = -2
c = 1
x = seq(-5, 5, by = 0.1)
eta = a + b*x + c*x^2
eta_2 = a + b*x
y = exp(eta) / (1 + exp(eta))
y_2 = exp(eta_2) / (1 + exp(eta_2))
matplot(x, cbind(y, y_2), type = "l", col = 1:2, lty = 1, bty = "l")
legend("bottomleft", legend = c("linear", "quadratic"),
       lty = 1, col = 1:2, bty = "n")

### 3.1
# See separate file.

### 3.2
# Y = f(x) = beta_0 + beta_1*x + beta_2*x^2 + ... + beta_{p-1}*x^{p-1} 
#   = [1 x x^2 ... x^{p-1}][beta_0 beta_1 beta_2 ... beta_{p-1}]^T 
#   = X*beta
# Fit this with least squares and get \hat{beta}, and \hat{Y} = X\hat{beta}.
# From page 249-250, we get that E[\hat{Y}] = Y = f(x) and Var(\hat{Y}) = sigma^2*P,
# where P = X(X^TX)^{-1}X^T.
# Then we can insert this into (3.2).
# 1)
# (E[\hat{Y}]] - f(x))^2 = (f(x) - f(x))^2 = 0.
# That is, if the underlying model is a p degree polynomial and we estimate it
# with a d degree polynomial we get zero bias.
# 2)
# Just use Var(\hat{Y}) = sigma^2*P.

### 3.3
# See separate file.

### 3.4
# See separate file.

### Exercise related to MSE
# Clear workspace
rm(list=ls())

# Get the data and create the polynomials
dataset = read.table("http://azzalini.stat.unipd.it/Book-DM/yesterday.dat", header = TRUE)
attach(dataset)
head(dataset)
n <- nrow(dataset)
y.range <- range(c(y.yesterday, y.tomorrow))
polinomio <- function(p) if(p==0) lm(y.yesterday~1) else lm(y.yesterday~poly(x,p))
polinomi <- apply(cbind(0:23), 1, polinomio)
y.hat <- lapply(polinomi, predict)

# Closer to diagonal when we increase order.
# That means that the predicted values are
# close to the true trainings response
plot(y.yesterday, y.hat[[4]])
plot(y.yesterday, y.hat[[15]])
plot(y.yesterday, y.hat[[23]])

#1) mse based on training
mse_yesterday = function(y.hat) mean((y.yesterday-y.hat)^2)
mse_train <- as.numeric(lapply(y.hat, mse_yesterday))
plot(mse_train)

# Mse based on test
mse_tommorow = function( y.hat) mean((y.tomorrow-y.hat)^2)
mse_test <- as.numeric(lapply(y.hat, mse_tommorow))
plot(0:23, mse_train, type="b", xlab="p", ylab="mse", ylim=c(0,max(mse_train)))
lines(0:23, mse_test, type = "b", col = 2)
legend("topright", legend = c("train", "test", "train-test", "loocv"), lty = 1,
       col = 1:4)
# same as figure 3.8

# Mse based on both training and test
y.hatdob = lapply(1:24, function(j) rep(y.hat[[j]],2))
mse_together = function(y.hatdob) mean((c(y.yesterday, y.tomorrow)-y.hatdob)^2)
mse_train_and_test <- as.numeric(lapply(y.hatdob, mse_together))
lines(0:23, mse_train_and_test, type = "b", col = 3)

## leave on out cross validation
mse_loocv = rep(NA, 24)
# Algorithm 3.1, see page 55.
for (p in 0:22) {
  residuals = rep(NA, n)
  for (i in 1:n) {
    dFrame = data.frame('x' = x[-i], 'y' = y.yesterday[-i])
    dPred = data.frame('x' = x[i])
    if(p==0) {
      fit_now = lm(y~1, data = dFrame) 
    } else {
      fit_now = lm(y~poly(x,p), data = dFrame)
    }
    residuals[i] = y.yesterday[i] - predict(fit_now, dPred)
  } 
  mse_loocv[p] = mean(residuals^2)
}

# Plot them
plot(0:23, mse_train, type="b", xlab="p", ylab="mse", ylim=c(0,max(mse_train)))
lines(0:23, mse_test, type = "b", col = 2)
lines(0:23, mse_train_and_test, type = "b", col = 3)
lines(0:23, mse_loocv, col = 4, type = "b")
legend("topright", legend = c("train", "test", "train-test", "loocv"), lty = 1,
       col = 1:4)









