# Programs used for graphics and tables of the book:
#    "Data anlysis and data mining" by A.Azzalini and B.Scarpa,
#     © Oxford University Press, 2012 (ISBN 978-0-19-976710-6).
#
# Code regarding section 3.2 (© 2003, 2004, 2012 A.Azzalini and B.Scarpa)
#------------------------------------------------------------------------

source("base-www.R")
dataset <- read.table("yesterday.dat", header=TRUE)
dataset = read.table("http://azzalini.stat.unipd.it/Book-DM/yesterday.dat", header = TRUE)
attach(dataset)
n<- nrow(dataset)
y.range<- range(c(y.yesterday, y.tomorrow))

polinomio <- function(p) if(p==0) lm(y.yesterday~1) else lm(y.yesterday~poly(x,p))
polinomi <- apply(cbind(0:23), 1, polinomio)


x0<- data.frame(x=sort(unique(c(x,seq(min(x), max(x), length=200)))))

#----training set----

name <- "figure 3.1"
plot(x, y.yesterday, ylim=y.range, pch=16, ylab="y")
#pause(name)

name <- "figure 3.2a"
plot(x, y.yesterday, ylim=y.range, col=col1, pch=pch1, xlab="",
     main="data and 3rd degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[4]], newdata=x0))
#pause(name)

name <- "figure 3.2b"
plot(x, y.yesterday, ylim=y.range, col=col1, pch=pch1,  xlab="",
     main="data and 6th degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[7]], newdata=x0))
#pause(name)

name <- "figure 3.2c"
plot(x, y.yesterday, ylim=y.range, col=col1, pch=pch1,  xlab="",
     main="data and 12th degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[13]], newdata=x0))
#pause(name)

name <- "figure 3.2d"
plot(x, y.yesterday, ylim=y.range, col=col1, pch=pch1,  xlab="",
     main="data and 18th degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[19]], newdata=x0))
#pause(name)

#---
#with R 2.13 it is not possible to estimate a polynomial 
#of order bigger than 23.
#For polynomial of degree 24 we need to go back to R 1.91
#---
#name <- "figure 3.2e"
#plot(x, y,ylim=y.range, col=col.1, pch=pch.1, 
#     main="data and 24th degree polynomial", font.main=1)
#lines(x0$x, predict(polinomi[[25]], newdata=x0))
#pause(name)
#name <- "figure 3.2f"
#plot(x, y,ylim=y.range, col=col.1, pch=pch.1, 
#     main="Data and (n-1)th Degree Polynomial")
#lines(x0$x, predict(polinomi[[n]], newdata=x0))
#pause(name)
#
#...we use data obtained by other tools
#

#------
#we only can obtain errors up to the degree 23
name <- "figure 3.3a"
dev <- as.numeric(lapply( polinomi, deviance))
plot(0:(23), dev, type="b", xlab="p", ylab="Deviance", ylim=c(0,max(dev)))
#pause(name)

name <- "figure 3.3b"
R2 <- function(obj) summary(obj)$r.squared
r2 <-  as.numeric(lapply( polinomi, R2))
plot(0:(23), r2, type="b", xlab="p", ylab=expression(R^2))
#pause(name)


#-------------------------------------------
#------ test set ---------------------

name <- "figure 3.4a"
plot(x, y.tomorrow, ylim=y.range, pch=pch2,  xlab="",
     main="data and 3rd degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[4]], newdata=x0))
#pause(name)

name <- "figure 3.4b"
plot(x, y.tomorrow, ylim=y.range, pch=pch2,  xlab="",
     main="data and 6th degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[7]], newdata=x0))
#pause(name)

name <- "figure 3.4c"
plot(x, y.tomorrow, ylim=y.range, pch=pch2,  xlab="",
     main="data and 12th degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[13]], newdata=x0))
#pause(name)

name <- "figure 3.4d"
plot(x, y.tomorrow, ylim=y.range, pch=pch2,  xlab="",
     main="data and 18th degree polynomial", font.main=1)
lines(x0$x, predict(polinomi[[19]], newdata=x0))
#pause(name)

#name <- "figure 3.4e"
#plot(x, y.tomorrow, ylim=y.range, pch=pch2, 
#     main="data and 24th degree polynomial", font.main=1)
#lines(x0$x, predict(polinomi[[25]], newdata=x0))
#pause(name)

#name <- "figure 3.4f"
#plot(x, y.tomorrow, ylim=y.range, pch=pch2, 
#     main="data and (n-1)th degree polynomial", font.main=1)
# lines(x0$x, predict(polinomi[[n]], newdata=x0))
#pause(name)

#------
#we only can obtain errors up to the degree 23

name<- "figure 3.5a"
y.hat <- lapply( polinomi, predict)
devianza <- function( y.hat) sum((y.tomorrow-y.hat)^2)
dev2 <- as.numeric(lapply(y.hat, devianza))
plot(0:(23), dev2, type="b", xlab="p", ylab="deviance", ylim=c(0,max(dev)))
#pause(name)


name <-"figure 3.5b"
r2 <- 1-dev2/dev[1]
plot(0:(23), r2, type="b", xlab="p", ylab=expression(R^2), ylim=c(0,1))
pause(name)

#detach.all()



#1) mse based on training
mse_yesterday = function( y.hat) mean((y.yesterday-y.hat)^2)
mse_train <- as.numeric(lapply(y.hat, mse_yesterday))
plot(0:(23), mse_train, type="b", xlab="p", ylab="mse", ylim=c(0,max(mse_train)))


mse_tommorow = function( y.hat) mean((y.tomorrow-y.hat)^2)
mse_test <- as.numeric(lapply(y.hat, mse_tommorow))
lines(0:23, mse_test, type = "b", col = 2)


y.hatdob = lapply(1:24, function(j) rep(y.hat[[j]],2))

c(y.yesterday, y.tomorrow)

mse_together = function(y.hatdob) mean((c(y.yesterday, y.tomorrow)-y.hatdob)^2)
mse_train_and_test <- as.numeric(lapply(y.hatdob, mse_together))
lines(0:23, mse_train_and_test, type = "b", col = 3)


## leave on out cross validation

require(boot)
?cv.glm

loocv=function(fit){
    h=lm.influence(fit)$h
    mean((residuals(fit)/(1-h))^2)
}

# The polynomial degree
degree=0:10

# A vector for collecting the errors.
cv.error=vector(mode="numeric",length=length(degree))

# A fit for each degree
for(d in degree){
    if (d == 0) {
        glm.fit = glm(y.yesterday ~ 1)
    } else {
        glm.fit=glm(y.yesterday~poly(x,d))#, data=myDataFrame)
    }
    cv.error[d]=loocv(glm.fit)
}
# The plot of the errors
lines(degree,cv.error,type="b")


mse_loocv = rep(NA, 24)
# Alogithm 3.1
for (p in 0:22) {
    residuals = rep(NA, n)
    for (i in 1:n) {
        y_now = y.yesterday[-i]
        x_now = x[-i]
        
        y_out = y.yesterday[i]
        x_out = x[i]
        
        dFrame = data.frame('x' = x[-i], 'y' = y.yesterday[-i])
        dPred = data.frame('x' = x[i])
        
        if(p==0) {
            fit_now = lm(y~1, data = dFrame) 
        } else {
            fit_now = lm(y~poly(x,p), data = dFrame)
            
            #tt = matrix(data.frame(poly(x_now,p)), ncol = p, nrow = n-1)
            #fit_now = lm(y_now~tt)
        }
        
        residuals[i] = y_out - predict(fit_now, dPred)
        
        
    } 
    mse_loocv[p] = mean(residuals^2)
    
}
mse_loocv
lines(0:23, mse_loocv, col = 7, type = "b")

legend("topright", legend = c("train", "test", "train-test", "loocv"), lty = 1,
       col = 1:4)



#1) mse based on training
mse_yesterday = function( y.hat) mean((y.yesterday-y.hat)^2)
mse_train <- as.numeric(lapply(y.hat, mse_yesterday))

# Mse based on test
mse_tommorow = function( y.hat) mean((y.tomorrow-y.hat)^2)
mse_test <- as.numeric(lapply(y.hat, mse_tommorow))

# Mse baed on both training and test
y.hatdob = lapply(1:24, function(j) rep(y.hat[[j]],2))
mse_together = function(y.hatdob) mean((c(y.yesterday, y.tomorrow)-y.hatdob)^2)
mse_train_and_test <- as.numeric(lapply(y.hatdob, mse_together))

## leave on out cross validation
mse_loocv = rep(NA, 24)
# Alogithm 3.1
for (p in 0:22) {
    residuals = rep(NA, n)
    for (i in 1:n) {
        dFrame = data.frame('x' = x[-i], 'y' = y.yesterday[-i])
        dPred = data.frame('x' = x[i])
        
        if(p==0) {
            fit_now = lm(y~1, data = dFrame) 
        } else {
            fit_now = lm(y~poly(x,p), data = dFrame)
        }
        residuals[i] = y.yesterday[i] - predict(fit_now, dPred)
    } 
    mse_loocv[p] = mean(residuals^2)
}

# Plot them
plot(0:23, mse_train, type="b", xlab="p", ylab="mse", ylim=c(0,max(mse_train)))
lines(0:23, mse_test, type = "b", col = 2)
lines(0:23, mse_train_and_test, type = "b", col = 3)
lines(0:23, mse_loocv, col = 4, type = "b")
legend("topright", legend = c("train", "test", "train-test", "loocv"), lty = 1,
       col = 1:4)
